# Generated by Django 3.2.6 on 2021-09-10 14:50

import functools

from django.db import migrations, models

import dash.utils


class Migration(migrations.Migration):

    dependencies = [
        ("dashblocks", "0008_auto_20210902_1125"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dashblock",
            name="image",
            field=models.ImageField(
                blank=True,
                help_text="Any image that should be displayed with this content block, optional",
                null=True,
                upload_to=functools.partial(dash.utils.generate_file_path, *("dashblocks",), **{}),
            ),
        ),
        migrations.AlterField(
            model_name="dashblockimage",
            name="image",
            field=models.ImageField(
                height_field="height",
                upload_to=functools.partial(dash.utils.generate_file_path, *("dashblock_images/",), **{}),
                width_field="width",
            ),
        ),
    ]
