# Generated by Django 3.2.6 on 2021-10-25 15:04

import functools

from django.db import migrations, models

import dash.utils


class Migration(migrations.Migration):

    dependencies = [
        ("orgs", "0028_alter_org_config"),
    ]

    operations = [
        migrations.AlterField(
            model_name="org",
            name="logo",
            field=models.ImageField(
                blank=True,
                help_text="The logo that should be used for this organization",
                null=True,
                upload_to=functools.partial(dash.utils.generate_file_path, *("logos",), **{}),
            ),
        ),
        migrations.AlterField(
            model_name="orgbackground",
            name="image",
            field=models.ImageField(
                help_text="The image file",
                upload_to=functools.partial(dash.utils.generate_file_path, *("org_bgs",), **{}),
            ),
        ),
    ]
