# Generated by Django 3.2.6 on 2021-09-02 11:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("categories", "0007_auto_20170301_0914"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="category",
            options={"ordering": ["name"], "verbose_name_plural": "Categories"},
        ),
    ]
